@@ -45,8 +45,6 @@
 #define MONITOR_CONTENTS_WIDTH 157
 #define MONITOR_CONTENTS_HEIGHT 111
 
-#define PDEBUG(pix) (g_print ("file %s: line %d (%s): Setting pixbuf to %i %i\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, gdk_pixbuf_get_width (pix), gdk_pixbuf_get_height (pix)))
-
 static gboolean gdk_pixbuf_xlib_inited = FALSE;
 
 typedef struct _Renderer Renderer;
@@ -165,7 +163,7 @@ static void set_root_pixmap          (Pixmap pixmap);
 
 static gboolean is_nautilus_running  (void);
 
-static void output_compat_prefs (Preferences *prefs);
+static void output_compat_prefs      (const Preferences *prefs);
 
 guint
 applier_get_type (void)
@@ -302,8 +300,10 @@ applier_destroy (GtkObject *object)
 }
 
 void
-applier_apply_prefs (Applier *applier, Preferences *prefs,
-		     gboolean do_root, gboolean do_preview)
+applier_apply_prefs (Applier           *applier, 
+		     const Preferences *prefs,
+		     gboolean           do_root,
+		     gboolean           do_preview)
 {
 	Preferences *new_prefs;
 
@@ -315,7 +315,8 @@ applier_apply_prefs (Applier *applier, Preferences *prefs,
 	}
 
 	if (!prefs->enabled) {
-		draw_disabled_message (applier_get_preview_widget (applier));
+		if (do_preview)
+			draw_disabled_message (applier_get_preview_widget (applier));
 		return;
 	}
 
@@ -718,7 +719,6 @@ renderer_render_background (Renderer *renderer)
 						  (GdkPixbufDestroyNotify) 
 						  g_free, 
 						  NULL);
-		PDEBUG (renderer->pixbuf); 
 	}
 }
 
@@ -762,7 +762,7 @@ renderer_render_wallpaper (Renderer *renderer)
 			guint alpha_value;
 			guint32 colorv;
 
-			alpha_value = renderer->prefs->opacity;
+			alpha_value = 2.56 * renderer->prefs->opacity;
 			alpha_value = alpha_value * alpha_value / 256;
 			alpha_value = CLAMP (alpha_value, 0, 255);
 
@@ -827,7 +827,6 @@ renderer_render_wallpaper (Renderer *renderer)
 					 GDK_INTERP_BILINEAR,
 					 alpha_value, 65536,
 					 colorv, colorv);
-		PDEBUG (renderer->pixbuf); 
 			}
 		}
 		else if (renderer->wwidth != renderer->pwidth ||
@@ -881,7 +880,6 @@ renderer_render_wallpaper (Renderer *renderer)
 					gdk_pixbuf_unref (renderer->pixbuf);
 
 				renderer->pixbuf = renderer->wallpaper_pixbuf;
-		PDEBUG (renderer->pixbuf); 
 
 				gdk_pixbuf_ref (renderer->pixbuf);
 			}
@@ -1381,15 +1379,16 @@ is_nautilus_running (void)
 
 
 static void
-output_compat_prefs (Preferences *prefs)
+output_compat_prefs (const Preferences *prefs)
 {
+	static const gint wallpaper_types[] = { 0, 1, 3, 2 };
 	gchar *color;
-	
+
 	gnome_config_pop_prefix ();
-	gnome_config_set_bool ("/Background/Default/Enabled", prefs->wallpaper_enabled);
+	gnome_config_set_bool ("/Background/Default/Enabled", prefs->enabled);
 	gnome_config_set_string ("/Background/Default/wallpaper",
 				 (prefs->wallpaper_filename) ? prefs->wallpaper_filename : "none");
-	gnome_config_set_int ("/Background/Default/wallpaperAlign", prefs->wallpaper_type);
+	gnome_config_set_int ("/Background/Default/wallpaperAlign", wallpaper_types[prefs->wallpaper_type]);
 
 	color = g_strdup_printf ("#%02x%02x%02x",
 		prefs->color1->red >> 8,
